using System;
using System.Collections.Generic;
using System.Text;

namespace TimeZoneConversion
{
    class Program
    {
        // Demonstrates using TimeZoneInfo to convert to/from UTC, and between two time zones
        // using UTC as an interim step.  Shows the real differences between two time zones as
        // the time change occurs, and how .NET handles the overlapping hour when the clock
        // rolls back to Standard Time.
        //
        // The time zones in this example are Eastern and Pacific Time, and can adjusted to anything.
        static void Main(string[] args)
        {
            TimeZone zone = TimeZone.CurrentTimeZone;
            DateTime dt1 = new DateTime(2008, 1, 1);
            DateTime dt2 = new DateTime(2008, 6, 1);
            Console.WriteLine(zone.IsDaylightSavingTime(dt1));
            Console.WriteLine(zone.IsDaylightSavingTime(dt2));
            Console.WriteLine(zone.GetUtcOffset(dt1));
            Console.WriteLine(zone.GetUtcOffset(dt2));

            TimeZoneInfo zone1 = TimeZoneInfo.Local;
            Console.WriteLine(zone1.StandardName);
            Console.WriteLine(zone1.DaylightName);

            string Zone_ID = "Eastern Standard Time";
            foreach (TimeZoneInfo z in TimeZoneInfo.GetSystemTimeZones())
            {
                Console.WriteLine(z.Id);
                if (z.Id == Zone_ID) Console.WriteLine("********************");
            }

            Console.WriteLine();
            Console.WriteLine("******************** Convert UTC to time zone in Zone_ID: {0}", Zone_ID);

            string[] TestTimes1 = new string[] {
                "3/8/2009 05:59:59",  // last minute before Daylight Savings Time switch
                "3/8/2009 06:00:00",  // first minute of Daylight Savings Time switch
                "3/8/2009 06:59:59",
                "3/8/2009 07:00:00",
                "11/1/2009 05:59:00", // last minute before Standard Time switch
                "11/1/2009 06:00:00", // last minute before Standard Time switch
                "11/1/2009 06:59:00",
                "11/1/2009 07:00:00"
            };

            foreach (string timestamp in TestTimes1)
            {
                DateTime TestTime = DateTime.Parse(timestamp);  // UTC
                DateTime Zoned = TimeZoneInfo.ConvertTimeFromUtc(TestTime, TimeZoneInfo.FindSystemTimeZoneById(Zone_ID));
                Console.WriteLine("UTC: {0} = EST/EDT: {1}", TestTime, Zoned);
            }

            // Convert between two time zones, using UTC as the axis.  Note the effect during the 
            // 2:00AM time change between zones.
            TimeZoneInfo EasternTimeZone = TimeZoneInfo.FindSystemTimeZoneById("Eastern Standard Time");
            Console.WriteLine(EasternTimeZone.StandardName);
            Console.WriteLine(EasternTimeZone.DaylightName);
            TimeZoneInfo PacificTimeZone = TimeZoneInfo.FindSystemTimeZoneById("Pacific Standard Time");
            Console.WriteLine(PacificTimeZone.StandardName);
            Console.WriteLine(PacificTimeZone.DaylightName);

            Console.WriteLine();
            Console.WriteLine("******************** Convert Eastern Time to Pacific Time");

            string[] TestTimes2 = new string[] {
                "3/8/2009 01:59:59",  // last minute before Daylight Savings Time switch
                "3/8/2009 03:00:00",  // first minute of Daylight Savings Time switch.. 4 hours ahead of PST
                "3/8/2009 04:00:00",  // 4 hours ahead of PST
                "3/8/2009 05:00:00",  // 4 hours ahead of PST
                "3/8/2009 06:00:00",  // first minute of PDT, now 3 hour difference resumes.
                "11/1/2009 00:59:00", // last minute before Standard Time switch
                "11/1/2009 01:00:00", // first minute of Daylight Savings Time switch.. 2 hours ahead of PST
                "11/1/2009 02:00:00", // 2 hours ahead of PDT
                "11/1/2009 03:00:00", // 2 hours ahead of PDT
                "11/1/2009 04:00:00"  // first minute of PST, now 3 hour difference resumes.
            };

            // So here is the ambiguity: switching from Daylight Savings Time to Standard Time
            // causes the 1AM hour to occur twice on the day of the switch.  The TimeZoneInfo
            // object will treat it as the 2nd occurrence.

            // Note: 02:00:00 to 02:59:59 on the day when the clock is moved ahead (i.e.
            // Standard to Daylight Savings switch) is illegal.  This hour is lost when switching
            // to Daylight Savings from Standard, and will throw an error.
            
            foreach (string timestamp in TestTimes2)
            {
                DateTime TestTime = DateTime.Parse(timestamp);  // Eastern Time
                DateTime UTC = TimeZoneInfo.ConvertTimeToUtc(TestTime, EasternTimeZone);
                DateTime Pacific = TimeZoneInfo.ConvertTimeFromUtc(UTC, PacificTimeZone);
                Console.WriteLine("ExT: {0} UTC: {1} PxT: {2}", TestTime, UTC, Pacific);
            }

#if DEBUG
            Console.ReadLine();
#endif
        }
    }
}
